@@ -12,6 +12,7 @@ modules = [
     "2chan",
     "35photo",
     "3dbooru",
+    "420chan",
     "4chan",
     "500px",
     "8kun",
@@ -20,11 +21,14 @@ modules = [
     "architizer",
     "artstation",
     "aryion",
+    "bbc",
     "bcy",
     "behance",
     "blogger",
+    "comicvine",
     "cyberdrop",
     "danbooru",
+    "desktopography",
     "deviantart",
     "dynastyscans",
     "e621",
@@ -74,6 +78,7 @@ modules = [
     "mangakakalot",
     "manganelo",
     "mangapark",
+    "mangasee",
     "mangoxo",
     "myhentaigallery",
     "myportfolio",
@@ -105,6 +110,7 @@ modules = [
     "sankaku",
     "sankakucomplex",
     "seiga",
+    "seisoparty",
     "senmanga",
     "sexcom",
     "simplyhentai",
@@ -128,6 +134,7 @@ modules = [
     "webtoons",
     "weibo",
     "wikiart",
+    "wikieat",
     "xhamster",
     "xvideos",
     "booru",
@@ -141,6 +148,7 @@ modules = [
     "recursive",
     "oauth",
     "test",
+    "ytdl",
 ]
 
 
