@@ -122,7 +122,7 @@ class ExhentaiGalleryExtractor(ExhentaiExtractor):
                 "date": "dt:2018-03-18 20:15:00",
                 "eh_category": "Non-H",
                 "expunged": False,
-                "favorites": "17",
+                "favorites": "18",
                 "filecount": "4",
                 "filesize": 1488978,
                 "gid": 1200119,
@@ -203,8 +203,8 @@ class ExhentaiGalleryExtractor(ExhentaiExtractor):
 
         def _validate_response(response):
             # declared inside 'items()' to be able to access 'data'
-            if not response.history and \
-                    response.headers.get("content-length") == "137":
+            if not response.history and response.headers.get(
+                    "content-type", "").startswith("text/html"):
                 self._report_limits(data)
             return True
 
