@@ -9,7 +9,9 @@
 """Extractors for https://kemono.party/"""
 
 from .common import Extractor, Message
-from .. import text
+from .. import text, exception
+from ..cache import cache
+import itertools
 import re
 
 BASE_PATTERN = r"(?:https?://)?kemono\.party/([^/?#]+)/user/([^/?#]+)"
@@ -22,17 +24,28 @@ class KemonopartyExtractor(Extractor):
     directory_fmt = ("{category}", "{service}", "{user}")
     filename_fmt = "{id}_{title}_{num:>02}_{filename}.{extension}"
     archive_fmt = "{service}_{user}_{id}_{num}"
+    cookiedomain = ".kemono.party"
 
     def items(self):
+        self._prepare_ddosguard_cookies()
+
         find_inline = re.compile(r'src="(/inline/[^"]+)').findall
+        skip_service = \
+            "patreon" if self.config("patreon-skip-file", True) else None
 
         if self.config("metadata"):
             username = text.unescape(text.extract(
-                self.request(self.user_url).text, "<title>", " | Kemono<")[0])
+                self.request(self.user_url).text,
+                '<meta name="artist_name" content="', '"')[0])
         else:
             username = None
 
-        for post in self.posts():
+        posts = self.posts()
+        max_posts = self.config("max-posts")
+        if max_posts:
+            posts = itertools.islice(posts, max_posts)
+
+        for post in posts:
 
             files = []
             append = files.append
@@ -40,7 +53,8 @@ class KemonopartyExtractor(Extractor):
 
             if file:
                 file["type"] = "file"
-                append(file)
+                if post["service"] != skip_service or not post["attachments"]:
+                    append(file)
             for attachment in post["attachments"]:
                 attachment["type"] = "attachment"
                 append(attachment)
@@ -57,35 +71,60 @@ class KemonopartyExtractor(Extractor):
                 post["type"] = file["type"]
                 url = file["path"]
                 if url[0] == "/":
-                    url = "https://data.kemono.party" + url
-                elif url.startswith("https://kemono.party/"):
-                    url = "https://data.kemono.party" + url[20:]
+                    url = self.root + "/data" + url
+                elif url.startswith("https://kemono.party"):
+                    url = self.root + "/data" + url[20:]
 
                 text.nameext_from_url(file["name"], post)
                 yield Message.Url, url, post
 
+    def login(self):
+        username, password = self._get_auth_info()
+        if username:
+            self._update_cookies(self._login_impl(username, password))
+
+    @cache(maxage=28*24*3600, keyarg=1)
+    def _login_impl(self, username, password):
+        self.log.info("Logging in as %s", username)
+
+        url = self.root + "/account/login"
+        data = {"username": username, "password": password}
+
+        response = self.request(url, method="POST", data=data)
+        if response.url.endswith("/account/login") and \
+                "Username or password is incorrect" in response.text:
+            raise exception.AuthenticationError()
+
+        return {c.name: c.value for c in response.history[0].cookies}
+
 
 class KemonopartyUserExtractor(KemonopartyExtractor):
     """Extractor for all posts from a kemono.party user listing"""
     subcategory = "user"
-    pattern = BASE_PATTERN + r"/?(?:$|[?#])"
+    pattern = BASE_PATTERN + r"/?(?:\?o=(\d+))?(?:$|[?#])"
     test = (
         ("https://kemono.party/fanbox/user/6993449", {
             "range": "1-25",
             "count": 25,
         }),
+        # 'max-posts' option, 'o' query parameter (#1674)
+        ("https://kemono.party/patreon/user/881792?o=150", {
+            "options": (("max-posts", 25),),
+            "count": "< 100",
+        }),
         ("https://kemono.party/subscribestar/user/alcorart"),
     )
 
     def __init__(self, match):
         KemonopartyExtractor.__init__(self, match)
-        service, user_id = match.groups()
+        service, user_id, offset = match.groups()
         self.api_url = "{}/api/{}/user/{}".format(self.root, service, user_id)
         self.user_url = "{}/{}/user/{}".format(self.root, service, user_id)
+        self.offset = text.parse_int(offset)
 
     def posts(self):
         url = self.api_url
-        params = {"o": 0}
+        params = {"o": self.offset}
 
         while True:
             posts = self.request(url, params=params).json()
@@ -102,7 +141,7 @@ class KemonopartyPostExtractor(KemonopartyExtractor):
     pattern = BASE_PATTERN + r"/post/([^/?#]+)"
     test = (
         ("https://kemono.party/fanbox/user/6993449/post/506575", {
-            "pattern": r"https://data\.kemono\.party/files/fanbox"
+            "pattern": r"https://kemono\.party/data/files/fanbox"
                        r"/6993449/506575/P058kDFYus7DbqAkGlfWTlOr\.jpeg",
             "keyword": {
                 "added": "Wed, 06 May 2020 20:28:02 GMT",
@@ -125,14 +164,24 @@ class KemonopartyPostExtractor(KemonopartyExtractor):
         }),
         # inline image (#1286)
         ("https://kemono.party/fanbox/user/7356311/post/802343", {
-            "pattern": r"https://data\.kemono\.party/inline/fanbox"
+            "pattern": r"https://kemono\.party/data/inline/fanbox"
                        r"/uaozO4Yga6ydkGIJFAQDixfE\.jpeg",
         }),
         # kemono.party -> data.kemono.party
         ("https://kemono.party/gumroad/user/trylsc/post/IURjT", {
-            "pattern": r"https://data\.kemono\.party/(file|attachment)s"
+            "pattern": r"https://kemono\.party/data/(file|attachment)s"
                        r"/gumroad/trylsc/IURjT/",
         }),
+        # username (#1548, #1652)
+        ("https://kemono.party/gumroad/user/3252870377455/post/aJnAH", {
+            "options": (("metadata", True),),
+            "keyword": {"username": "Kudalyn's Creations"},
+        }),
+        # skip patreon main file (#1667, #1689)
+        ("https://kemono.party/patreon/user/4158582/post/32099982", {
+            "count": 2,
+            "keyword": {"type": "attachment"},
+        }),
         ("https://kemono.party/subscribestar/user/alcorart/post/184330"),
     )
 
@@ -146,3 +195,25 @@ class KemonopartyPostExtractor(KemonopartyExtractor):
     def posts(self):
         posts = self.request(self.api_url).json()
         return (posts[0],) if len(posts) > 1 else posts
+
+
+class KemonopartyFavoriteExtractor(KemonopartyExtractor):
+    """Extractor for kemono.party favorites"""
+    subcategory = "favorite"
+    pattern = r"(?:https?://)?kemono\.party/favorites"
+    test = ("https://kemono.party/favorites", {
+        "pattern": KemonopartyUserExtractor.pattern,
+        "url": "f4b5b796979bcba824af84206578c79101c7f0e1",
+        "count": 3,
+    })
+
+    def items(self):
+        self._prepare_ddosguard_cookies()
+        self.login()
+
+        users = self.request(self.root + "/api/favorites").json()
+        for user in users:
+            user["_extractor"] = KemonopartyUserExtractor
+            url = "{}/{}/user/{}".format(
+                self.root, user["service"], user["id"])
+            yield Message.Queue, url, user
