@@ -72,5 +72,3 @@ class RedgifsImageExtractor(RedgifsExtractor):
 
 class RedgifsAPI(GfycatAPI):
     API_ROOT = "https://api.redgifs.com"
-    ACCESS_KEY = ("dBLwVuGn9eq4dtXLs8WSfpjcYFY7bPQe"
-                  "AqGPSFgqeW5B9uzj2cMVhF63pTFF4Rg9")
