@@ -15,7 +15,7 @@ import operator
 import functools
 import collections
 from . import extractor, downloader, postprocessor
-from . import config, text, util, output, exception
+from . import config, text, util, path, formatter, output, exception
 from .extractor.message import Message
 
 
@@ -33,6 +33,7 @@ class Job():
         self.pathfmt = None
         self.kwdict = {}
         self.status = 0
+        self.url_key = extr.config("url-metadata")
 
         self._logger_extra = {
             "job"      : self,
@@ -57,7 +58,7 @@ class Job():
             extr.session.adapters = pextr.session.adapters
 
         # user-supplied metadata
-        kwdict = self.extractor.config("keywords")
+        kwdict = extr.config("keywords")
         if kwdict:
             self.kwdict.update(kwdict)
 
@@ -67,12 +68,16 @@ class Job():
 
     def run(self):
         """Execute or run the job"""
-        sleep = self.extractor.config("sleep-extractor")
+        extractor = self.extractor
+        log = extractor.log
+        msg = None
+
+        sleep = util.build_duration_func(extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep)
+            time.sleep(sleep())
+
         try:
-            log = self.extractor.log
-            for msg in self.extractor:
+            for msg in extractor:
                 self.dispatch(msg)
         except exception.StopExtraction as exc:
             if exc.message:
@@ -99,33 +104,36 @@ class Job():
         except BaseException:
             self.status |= 1
             raise
+        else:
+            if msg is None:
+                log.info("No results for %s", extractor.url)
         finally:
             self.handle_finalize()
+            if extractor.finalize:
+                extractor.finalize()
+
         return self.status
 
     def dispatch(self, msg):
         """Call the appropriate message handler"""
         if msg[0] == Message.Url:
-            _, url, kwds = msg
-            if self.pred_url(url, kwds):
-                self.update_kwdict(kwds)
-                self.handle_url(url, kwds)
+            _, url, kwdict = msg
+            if self.url_key:
+                kwdict[self.url_key] = url
+            if self.pred_url(url, kwdict):
+                self.update_kwdict(kwdict)
+                self.handle_url(url, kwdict)
 
         elif msg[0] == Message.Directory:
             self.update_kwdict(msg[1])
             self.handle_directory(msg[1])
 
         elif msg[0] == Message.Queue:
-            _, url, kwds = msg
-            if self.pred_queue(url, kwds):
-                self.handle_queue(url, kwds)
-
-        elif msg[0] == Message.Version:
-            if msg[1] != 1:
-                raise "unsupported message-version ({}, {})".format(
-                    self.extractor.category, msg[1]
-                )
-            # TODO: support for multiple message versions
+            _, url, kwdict = msg
+            if self.url_key:
+                kwdict[self.url_key] = url
+            if self.pred_queue(url, kwdict):
+                self.handle_queue(url, kwdict)
 
     def handle_url(self, url, kwdict):
         """Handle Message.Url"""
@@ -194,6 +202,7 @@ class DownloadJob(Job):
         Job.__init__(self, url, parent)
         self.log = self.get_logger("download")
         self.blacklist = None
+        self.fallback = None
         self.archive = None
         self.sleep = None
         self.hooks = ()
@@ -227,13 +236,14 @@ class DownloadJob(Job):
             return
 
         if self.sleep:
-            time.sleep(self.sleep)
+            time.sleep(self.sleep())
 
         # download from URL
         if not self.download(url):
 
-            # use fallback URLs if available
-            for num, url in enumerate(kwdict.get("_fallback", ()), 1):
+            # use fallback URLs if available/enabled
+            fallback = kwdict.get("_fallback", ()) if self.fallback else ()
+            for num, url in enumerate(fallback, 1):
                 util.remove_file(pathfmt.temppath)
                 self.log.info("Trying fallback URL #%d", num)
                 if self.download(url):
@@ -302,11 +312,18 @@ class DownloadJob(Job):
             else:
                 extr._parentdir = pextr._parentdir
 
-            if pextr.config("parent-metadata"):
-                if self.kwdict:
-                    job.kwdict.update(self.kwdict)
-                if kwdict:
-                    job.kwdict.update(kwdict)
+            pmeta = pextr.config("parent-metadata")
+            if pmeta:
+                if isinstance(pmeta, str):
+                    data = self.kwdict.copy()
+                    if kwdict:
+                        data.update(kwdict)
+                    job.kwdict[pmeta] = data
+                else:
+                    if self.kwdict:
+                        job.kwdict.update(self.kwdict)
+                    if kwdict:
+                        job.kwdict.update(kwdict)
 
             if pextr.config("parent-skip"):
                 job._skipcnt = self._skipcnt
@@ -377,28 +394,30 @@ class DownloadJob(Job):
     def initialize(self, kwdict=None):
         """Delayed initialization of PathFormat, etc."""
         cfg = self.extractor.config
-        pathfmt = self.pathfmt = util.PathFormat(self.extractor)
+        pathfmt = self.pathfmt = path.PathFormat(self.extractor)
         if kwdict:
             pathfmt.set_directory(kwdict)
 
-        self.sleep = cfg("sleep")
+        self.sleep = util.build_duration_func(cfg("sleep"))
+        self.fallback = cfg("fallback", True)
         if not cfg("download", True):
             # monkey-patch method to do nothing and always return True
             self.download = pathfmt.fix_extension
 
         archive = cfg("archive")
         if archive:
-            path = util.expand_path(archive)
+            archive = util.expand_path(archive)
             try:
-                if "{" in path:
-                    path = util.Formatter(path).format_map(kwdict)
-                self.archive = util.DownloadArchive(path, self.extractor)
+                if "{" in archive:
+                    archive = formatter.parse(archive).format_map(kwdict)
+                self.archive = util.DownloadArchive(archive, self.extractor)
             except Exception as exc:
                 self.extractor.log.warning(
                     "Failed to open download archive at '%s' ('%s: %s')",
-                    path, exc.__class__.__name__, exc)
+                    archive, exc.__class__.__name__, exc)
             else:
-                self.extractor.log.debug("Using download archive '%s'", path)
+                self.extractor.log.debug(
+                    "Using download archive '%s'", archive)
 
         skip = cfg("skip", True)
         if skip:
@@ -453,6 +472,7 @@ class DownloadJob(Job):
                 except Exception as exc:
                     pp_log.error("'%s' initialization failed:  %s: %s",
                                  name, exc.__class__.__name__, exc)
+                    pp_log.debug("", exc_info=True)
                 else:
                     pp_list.append(pp_obj)
 
@@ -523,7 +543,7 @@ class SimulationJob(DownloadJob):
         self.pathfmt.set_filename(kwdict)
         self.out.skip(self.pathfmt.path)
         if self.sleep:
-            time.sleep(self.sleep)
+            time.sleep(self.sleep())
         if self.archive:
             self.archive.add(kwdict)
 
@@ -626,7 +646,7 @@ class UrlJob(Job):
             extr = extractor.find(url)
 
         if extr:
-            self.status |= self.__class__(extr, self).run()
+            self.status |= self.__class__(extr, self, self.depth + 1).run()
         else:
             self._write_unsupported(url)
 
@@ -677,9 +697,10 @@ class DataJob(Job):
         self.filter = util.identity if private else util.filter_dict
 
     def run(self):
-        sleep = self.extractor.config("sleep-extractor")
+        sleep = util.build_duration_func(
+            self.extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep)
+            time.sleep(sleep())
 
         # collect data
         try:
